

/*
 * TalkServerHandler.java
 * The Server for the chat room program.
 * Receives commands from the client,
 * Executes the commands and
 * sends output back to the client
 *
 * Created on September 15, 2003, 11:40 PM
 */

/**
 *
 * @author  Nathan Balon
 * SN 3210 1717
 * CIS 527
 * Program Assignment #1
 * *
 */


import java.io.*;
import java.net.*;
import java.util.*;

/** TalkServerHandler implements the Server portion of the chat program */
public class TalkServerHandler extends Thread{

    private Socket clientSocket;                           // client socket
    private static int threadCount = 0;                    // count 0f threads
    private static HashMap loggedInUsers = new UserMap();  // logged in users
    private static ArrayList chatters = new ArrayList();   // array of logged in users
    private static UserMap userMap = new UserMap();        // map of registered users
     
    
    /** Creates a new instance of talkServer */
    public TalkServerHandler(Socket incoming, int count) {
        clientSocket = incoming;
        threadCount = count; 
        this.threadCount++;
        
    }
    
    public void run(){ 
        try{    
           // open the userList file with users and passwords
        try{
            userMap = (UserMap)userMap.open();
        }

        // if the file does not yet exist create the default file
        catch (FileNotFoundException e){
            userMap.createDefaultUsers();
        }

        // catch other exceptions due to open the userList file
        catch (Exception e){
            System.err.println("Unexpected ERROR: " + e);
        }
 
              // create a BufferedReader to read input streams from the client.
              BufferedReader is = new BufferedReader
                   (new InputStreamReader(clientSocket.getInputStream()));
              //create a PrintWriter to write output streams to the client.
              PrintWriter os = new PrintWriter
                (clientSocket.getOutputStream(), true);
           
              String inputStream = "";       // input recieved from the client
              String clientCommand = "";     // command to execute
           
              // loop to read strings from the client 
              // and process the correct commands.
              while((inputStream = is.readLine()) != null){ 

                   StringTokenizer tok = new StringTokenizer(inputStream);
                   // get the clients command
                   String clientInput = tok.nextToken();  
                   // convert the command to lower case
                   clientInput = clientInput.toLowerCase();
                
                   // process the send command
                   if(clientInput.equals("send")){
                       String senderName = "";
                       String receiverName = "";
                       String message = "";

                       if(tok.hasMoreTokens()){
                           receiverName = tok.nextToken();
                       }                    
                       if(tok.hasMoreTokens()){
                           senderName = tok.nextToken();
                       }
 
                       // get the message to be sent
                       while(tok.hasMoreTokens()){
                           message += tok.nextToken() + " ";
                       }                      
                           
                       // display message on the server
                       System.out.println("SERVER# send From: " + senderName + " To: "
                                           + receiverName + " " + message); 

                       // if receiver name equals all send the message to all users
                       if(receiverName.equals("all")){
                           for(int i = 0; i < chatters.size(); i++){
                               LoggedInUser u = (LoggedInUser)chatters.get(i);
                               PrintWriter pw = new PrintWriter
                                    (u.getUserSocket().getOutputStream(), true);
                               pw.println("SERVER# " + senderName + ": " + message);
                               if(u.getName().equals(receiverName)){
                                   chatters.remove(i);
                               }
                           }
                       }
                       // send the message to a logged in user
                       else if(loggedInUsers.containsKey(receiverName)){                         
                          LoggedInUser userToSend = (LoggedInUser)loggedInUsers.get(receiverName);
                          // get the socket to send the message to
                          PrintWriter pw = new PrintWriter
                              (userToSend.getUserSocket().getOutputStream(), true);
                          // send message to the client                          
                          pw.println("SERVER# " + senderName + ": " + message);
                      }
                      else{
                         System.out.println("SERVER# !!!User: " + receiverName + " not found!!!");
                         os.println("SERVER# !!!User: " + receiverName + " not found!!!");
                      }    
                   }
                   
                   else if(clientInput.equals("who")){
                       String message = "";
                       for(int i = 0; i < chatters.size(); i++){
                           LoggedInUser u = (LoggedInUser)chatters.get(i);
                           message += u.toString() + "  ";
                       }
                       System.out.println("SERVER# " + message);
                       os.println("SERVER# " + message);
                   }
                   
                   else if(clientInput.equals("new")){
                       String name = " ";      // new users name
                       String pass = " ";      // new user password
                    
                       // get the new users name and pass
                       name = tok.nextToken();
                       pass = tok.nextToken();
                    
                       // check if the user already exist
                       if(userMap.containsKey(name)){
                           os.println("SERVER# user already " +
                                      "has an account");
                       }
                       // add the user to the file
                       else{
                           // create a new user
                           User newUser = new User(name, pass);   
                           // add the user to the list of users
                           userMap.put(name, newUser);
                           // save the user to the file
                           userMap.save();
                           // display sucess message
                           System.out.println("SERVER# Added new user "
                                              + name);
                           os.println("SERVER# Added new user " + name );    
                       }
                   }
                
                   else if(clientInput.equals("login")){
                       String name = "";             // user name
                       String pass = "";             // user password
                    
                       // get the user name and password
                       if(tok.hasMoreTokens()){
                           name = tok.nextToken(); 
                       }
                       if(tok.hasMoreTokens()){      
                           pass = tok.nextToken();  
                       }     
                       
                       // check if user name is already logged in
                       if(loggedInUsers.containsKey(name)){
                           os.println("SERVER# " + name + " is already logged in");
                       }
                       // log the user in
                       else{
                           // check if user name exist in the user list
                           if(userMap.containsKey(name)){
                               // if name exists get the user
                               User u = (User)userMap.get(name);
                               // if the password match the user password
                               // log them in to chat
                               if(pass.equals(u.getPassword())){
                                   System.out.println("SERVER# " + name + 
                                               " joined the chat room"); 

                                   // put user in list of logged in users
                                   LoggedInUser loggedInUser =
                                        new LoggedInUser(u.getName(), 
                                            this.clientSocket.getInetAddress(), 
                                            this.clientSocket);
                                   chatters.add(loggedInUser);
                                   loggedInUsers.put(u.getName(), loggedInUser);   

                                   for(int i = 0; i < chatters.size(); i++){
                                       LoggedInUser uLogged = (LoggedInUser)chatters.get(i);
                                       PrintWriter pw = new PrintWriter
                                         (uLogged.getUserSocket().getOutputStream(), true);
                                       if(loggedInUser.getName().equals(uLogged.getName())){
                                           pw.println("true " + u.getName() + " " + 
                                           u.getPassword() + " SERVER# "
                                              + name + " joined the chat room");
                                       }
                                       else{
                                           pw.println("SERVER# " + name + 
                                                      " joined the chat room");
                                       } 
                                   }
                               }   
                               else{
                               // if name is in list but passwords don't match
                                   System.out.println("SERVER# User " + name + 
                                                      " incorrect password"); 
                                   os.println("false SERVER# User " + name + 
                                              " incorrect password"); 
                              }
                         
                          }
                          else{
                              // if the user can't be logged in because 
                              // they have registered
                              os.println("false SERVER# User not found " +
                                         "please register");
                              System.out.println("SERVER# User not found " +
                                                 "please register");
                         }
                    }
                }
                // log the user out of the chat room
                else if(clientInput.equals("logout")){
                    String name = tok.nextToken();
                    String pass = tok.nextToken();
                    // remove user from list of logged in users
                    loggedInUsers.remove(name);
 
                    for(int i = 0; i < chatters.size(); i++){
                        LoggedInUser uLogged = (LoggedInUser)chatters.get(i);
                        PrintWriter pw = new PrintWriter
                            (uLogged.getUserSocket().getOutputStream(), true);
                        pw.println("SERVER# " + name + " logged out of chat");
                    }
                    for(int i = 0; i < chatters.size(); i++){
                        LoggedInUser uLogged = (LoggedInUser)chatters.get(i);  
                        if(uLogged.getName().equals(name)){
                            chatters.remove(i);
                        }
                    }
                    System.out.println("SERVER# " + name +
                                       " logged out of chat");
                }
                // if user terminate chat
                else if(clientInput.equals("quit")){
                    System.out.println("SERVER# user quit chat");
                    // close the socket to the client
                    // clientSocket.close();
                }
                else{
                   System.out.println("SERVER# Unknown Command was entered");
                }
             
              }
              if(clientSocket != null){
                  clientSocket.close();
              }
          }
           
       
       catch (Exception e){
           e.printStackTrace();
       }
       finally{
           try{  
               if(clientSocket != null)
                   clientSocket.close();
           }
           catch (IOException e){
               System.err.println("IO Error: " + e);
           }
       }
    } 
}

